@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006-2013 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2006-2017 by Jakob Schröter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -28,7 +28,7 @@ namespace gloox
    * See @link gloox::VCardManager VCardManager @endlink for info on how to
    * fetch VCards.
    *
-   * @author Jakob Schroeter <js@camaya.net>
+   * @author Jakob Schröter <js@camaya.net>
    * @since 0.8
    */
   class GLOOX_API VCard : public StanzaExtension
@@ -428,7 +428,7 @@ namespace gloox
        * Sets a URI to a photo.
        * @param extval The URI to the photo.
        */
-      void setPhoto( const std::string& extval );
+      void setPhotoUri( const std::string& extval );
 
       /**
        * Sets the photo directly.
